//+------------------------------------------------------------------+
//|                              Candle-Reverse-Pattern-Fx-Tools.mq4 |
//+------------------------------------------------------------------+
#property link      "https://www.forexfactory.com/fx.tools"
#property version   "1.00"
#property strict
#property indicator_chart_window
#property indicator_buffers 2
#property indicator_plots   2
//--- plot BuySignal
#property indicator_label1  "BuySignal"
#property indicator_type1   DRAW_ARROW
#property indicator_color1  clrLimeGreen
#property indicator_style1  STYLE_SOLID
#property indicator_width1  1
//--- plot SellSignal
#property indicator_label2  "SellSignal"
#property indicator_type2   DRAW_ARROW
#property indicator_color2  clrRed
#property indicator_style2  STYLE_SOLID
#property indicator_width2  1

//--- input parameters
input bool     ShowAlert=true;

//--- indicator buffers
double         BuySignalBuffer[];
double         SellSignalBuffer[];

datetime LastAlertTime;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- indicator buffers mapping
   SetIndexBuffer(0,BuySignalBuffer);
   SetIndexBuffer(1,SellSignalBuffer);
//--- setting a code from the Wingdings charset as the property of PLOT_ARROW
   SetIndexArrow(0,233);
   SetIndexArrow(1,234);
   
   SetIndexEmptyValue(0, EMPTY_VALUE);
   SetIndexEmptyValue(1, EMPTY_VALUE);
   
   
   ArraySetAsSeries(BuySignalBuffer, true);
   ArraySetAsSeries(SellSignalBuffer, true);
         
   LastAlertTime = TimeCurrent()-PeriodSeconds();

//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//---
   int limit = rates_total;
   if (prev_calculated > 0) limit = rates_total-prev_calculated;
   if (limit >= rates_total - 2) limit = rates_total - 3;
   
   for (int i=limit; i>0; i--)
   {
      BuySignalBuffer[i] = EMPTY_VALUE;
      SellSignalBuffer[i] = EMPTY_VALUE;
      
      if (Open[i] <= Low[i+1] && Close[i] >= Close[i+1])
      {
         BuySignalBuffer[i] = Low[i] - AvRange(i, high, low, rates_total);
      }
      if (Open[i] >= High[i+1] && Close[i] <= Close[i+1])
      {
         SellSignalBuffer[i] = High[i] + AvRange(i, high, low, rates_total);
      }
   }
   
   if (ShowAlert && Time[0] > LastAlertTime)
   {  
      string msg = "";
      if (NormalizeDouble(BuySignalBuffer[1],Digits) != EMPTY_VALUE)
      {
         msg = WindowExpertName()+" BUY signal for "+_Symbol+" on "+EnumToString((ENUM_TIMEFRAMES)_Period)+" at "+TimeToStr(Time[1]);
         Alert(msg);
         LastAlertTime = Time[0];
      }
      else if (NormalizeDouble(SellSignalBuffer[1],Digits) != EMPTY_VALUE)
      {
         msg = WindowExpertName()+" SELL signal for "+_Symbol+" on "+EnumToString((ENUM_TIMEFRAMES)_Period)+" at "+TimeToStr(Time[1]);
         Alert(msg);
         LastAlertTime = Time[0];
      }         
   }
   
//--- return value of prev_calculated for next call
   return(rates_total);
  }
  
double AvRange(int pos, double const &high[], double const &low[], double rates_total)
{
   double sum=0;   
   int count = 4;
   int i=pos;
   
   while (i<rates_total && count > 0)
   {
      sum += high[i]-low[i];
      count--;
      i++;
   }
   return (sum/8);
}
//+------------------------------------------------------------------+
